@@ -1,8 +1,10 @@
 #!/usr/bin/env python3
 # encoding: utf-8
 
+from __future__ import annotations
+
 __author__ = "ChenyangGao <https://chenyanggao.github.io>"
-__version__ = (0, 1, 2)
+__version__ = (0, 1, 3)
 __all__ = [
     "AsyncExecutor", "get_loop", "stop_loop", "shutdown_loop", "run_loop", "submit", 
     "get_all_tasks", "cancel_all_tasks", "run_async", "run_in_loop", "as_thread", 
@@ -337,7 +339,7 @@ def ensure_aiter[T](
                 except StopIteration:
                     pass
     else:
-        if isinstance(iterable, Sequence):
+        if isinstance(iterable, Collection):
             threaded = False
         if threaded:
             async def wrapper():
@@ -498,25 +500,29 @@ async def async_zip(
         pass
 
 
-async def async_chain(
-    *iterables: Iterable | AsyncIterable, 
-    threaded: bool = False, 
-) -> AsyncIterator:
-    for iterable in iterables:
-        async for e in ensure_aiter(iterable, threaded=threaded):
-            yield e
+class async_chain[T](AsyncIterable):
 
+    def __init__(
+        self, 
+        /, 
+        *iterables: Iterable[T] | AsyncIterable[T], 
+        threaded: bool = False, 
+    ):
+        self.iterables = (ensure_aiter(it, threaded=threaded) for it in iterables)
 
-async def async_chain_from_iterable(
-    iterable: Iterable[Iterable | AsyncIterable] | AsyncIterable[Iterable | AsyncIterable], 
-    /, 
-    threaded: bool = False, 
-) -> AsyncIterator:
-    async for iterable in ensure_aiter(iterable, threaded=False):
-        async for e in ensure_aiter(iterable, threaded=threaded):
-            yield e
+    async def __aiter__(self, /) -> AsyncIterator[T]:
+        for iterable in self.iterables:
+            async for e in iterable:
+                yield e
 
-async_chain.from_iterable = async_chain_from_iterable # type: ignore
+    @classmethod
+    def from_iterable(
+        cls, 
+        /, 
+        iterables: Iterable[Iterable[T] | AsyncIterable[T]], 
+        threaded: bool = False, 
+    ) -> async_chain:
+        return cls(*iterables, threaded=threaded)
 
 
 async def async_all(
@@ -962,7 +968,7 @@ async def collect[K, V](
 async def collect[T](
     iterable: Iterable[T] | AsyncIterable[T], 
     /, 
-    rettype: Callable[[Iterable[T]], Collection[T]] = list, 
+    rettype: Callable[[Iterable[T]], Collection[T]] = list,  
     threaded: bool = False, 
 ) -> Collection[T]:
     ...
@@ -1001,5 +1007,5 @@ async def collect(
             async for k, v in iterable:
                 dt[k] = v
             return dt
-    return rettype([e async for e in iterable])
+    return cast(Callable, rettype)([e async for e in iterable])
 
